/*
 * Copyright (C) 2003-2015 FreeIPMI Core Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef IPMI_OEM_SPEC_H
#define IPMI_OEM_SPEC_H

#ifdef __cplusplus
extern "C" {
#endif

#warning ipmi-oem-spec.h header file is deprecated

#include <freeipmi/spec/oem/ipmi-oem-dell-spec.h>
#include <freeipmi/spec/oem/ipmi-oem-fujitsu-spec.h>
#include <freeipmi/spec/oem/ipmi-oem-ibm-spec.h>
#include <freeipmi/spec/oem/ipmi-oem-intel-spec.h>
#include <freeipmi/spec/oem/ipmi-oem-inventec-spec.h>
#include <freeipmi/spec/oem/ipmi-oem-quanta-spec.h>
#include <freeipmi/spec/oem/ipmi-oem-sun-microsystems-spec.h>
#include <freeipmi/spec/oem/ipmi-oem-supermicro-spec.h>
#include <freeipmi/spec/oem/ipmi-oem-wistron-spec.h>

#ifdef __cplusplus
}
#endif

#endif /* IPMI_OEM_SPEC_H */
